﻿/**
 * Sortd
 * Copyright (c) 2019, Sortd Inc
 * https://www.sortd.com
 *
 * @fileoverview Defines services to be used by Sortd extension
 * @name services
 */

'use strict';

/**
* Define the SortdEventPageErrorHandler module.
*/
var SortdEventPageErrorHandler = (function () {

    function SortdEventPageErrorHandler() {
    }

    /**
	 * sends a log error request to the content script e.g. the tab running the Sortd application
	 */
    SortdEventPageErrorHandler.prototype._sendLogErrorRequestToSortdApp = function (tabId, error) {

		try {
			if (error instanceof Error) {
				error = {
					message: error.message,
					stack: error.stack
				};
			}
			chrome.tabs.sendMessage(tabId, { action: 'logBackGroundError', data: { error: error } },

				function (response) {

					if (response) {
						console.log('Background Error logged...', error);
					}
				}
			);
		} catch (ex) {
			console.log('Could not send background error to tab');
		}
    };

    /**
	 * Sends a log error request to the content script
	 */
    SortdEventPageErrorHandler.prototype.reportBackgroundError = function (currentTabId, error) {

		var self = this;

		console.log('SortdEventPageErrorHandler', error);

        if (!currentTabId) {

        	try {
				chrome.tabs.query({'status': 'complete', url: 'https://mail.google.com/*'}, function (tabs) {

					if (tabs && tabs[0]) {
						self._sendLogErrorRequestToSortdApp(tabs[0].id, error);
					}
				});
			} catch (ex) {
				console.log('Could not call chrome.tabs.query in SortdEventPageErrorHandler', ex);
			}
        } else {
            self._sendLogErrorRequestToSortdApp(currentTabId, error);
        }
    };

    return SortdEventPageErrorHandler;
}());