/**
 * Sortd
 * Copyright (c) 2019, Sortd Inc
 * https://www.sortd.com
 *
 * @fileoverview Gmail Api wrapper
 * @name api
 */

class GmailApi {}

GmailApi.__FILE_NAME = 'api.js';

GmailApi.api = null;
GmailApi.initialized = false;
GmailApi.errorHandler = null;
GmailApi.messages = null;
GmailApi.hashedUser = null;

GmailApi.load = function(options) {

	GmailApi.config = options.config;
	GmailApi.errorHandler = options.errorHandler;
	GmailApi.messages = options.messages;
	GmailApi.tracker = options.tracker;

	return new Promise((resolve) => {

		if (GmailApi.api) {
			resolve(GmailApi.api);
			return;
		}

		const dom = new GmailApiDom({
			config: GmailApi.config
		});
			
		const notifications = new GmailApiNotifications({
			dom,
			config: GmailApi.config,
			errorHandler: GmailApi.errorHandler
		});
		notifications.init();

		const user = new GmailApiUser({
			config: GmailApi.config,
			errorHandler: GmailApi.errorHandler
		});

		GmailApi.api = {
			Dom: dom,
			User: user,
			Notifications: notifications,
			init: () => GmailApi.init()
		};

		resolve(GmailApi.api);
	});
}

GmailApi.init = function() {

	try {
		if (GmailApi.initialized) {
			return;
		}

		const sortdInboxSDK = new SortdInboxSdk({
			errorHandler: GmailApi.errorHandler,
			messages: GmailApi.messages,
			tracker: GmailApi.tracker,
			gmailApi: GmailApi.api
		});

		sortdInboxSDK.init()
			.then((sdk) => {
				GmailApi.api.User.inboxSDK = sdk;
			})
			.catch((err) => {
				GmailApi.errorHandler.handle(err, GmailApi.__FILE_NAME);
			});

		GmailApi.initialized = true;
	} catch (ex) {
		GmailApi.errorHandler.handle(ex, GmailApi.__FILE_NAME);
	}
}