/**
 * Sortd
 * Copyright (c) 2019, Sortd Inc
 * https://www.sortd.com
 *
 * @fileoverview Gmail Api DOM map
 * @name dom-map
 */

const SortdGmailApiDomMap = (function () {

	const domMap = localStorage.getItem(constants.STORAGE.KEYS.DOM_MAP);
	if (domMap) {
		return JSON.parse(domMap);
	} else {

        /**
         * IMPORTANT
         * This file is fallback only, it will be overwritten with domMap loaded from MongoDB (or the cached Redis version)
         * When updating this file, update DomMap in Admin CE page and DomMap in JSCE
         *
         * DO NOT ADD A "_FOUND" property on the same level as an "ELEMENT" property
         */

		return {

            /**
             * Z-indexes for various elements
             */
			"Z_INDEX": {
				"IFRAME": 7,
				"IFRAME_SIDEBAR": 4,
				"PREVIEW": 8,
				"SORTD_BUTTON": 10,
				"SORTD_DROPDOWN": 11,
				"POPUP_IFRAME": 15
			},

            /**
             * Accessed using domHelper.get, domHelper.getWithRetries or domHelper.wait
             * Each entry in array is a fallback for the same object
             * Each entry must contain "ELEMENT" key which is what the domHelper uses to check its existence
             * Related items should be added in one entry
             */
			"SORTD_FRAME": [
				{ "ELEMENT": ".sortd.frame" }
			],
			"COMPOSE_MAIL": [
				{
					"ELEMENT": ".dw"
				}
			],
			"GMAIL_COMPOSE_MAIL": [
				{ "ELEMENT": ".Yh.akV" },
				{ "ELEMENT": ".aic .T-I.T-I-KE.L3" }
			],
			"GMAIL_GOOGLE_APPS_MENU": [
				{
					"ELEMENT": ".gb_Ud .gb_C",
					"IFRAME": ".gb_Ud iframe"
				}
			],
			"GMAIL_IMAP_DETAILS_LINK": [
				{ "ELEMENT": ".l8.ou" }
			],
			"TOP_BAR": [
				{ "ELEMENT": "#gba" }
			],
			"MAIL_LIST_CONTAINER": [
				{ "ELEMENT": ".Tm.aeJ" }
			],
			"VIEW_MAIL": [
				{
					"ELEMENT": ".Bu .ade .hk:last-child img:visible",
					"POP_OUT_IMG": ".Bu .ade .hk:last-child img:visible",
					"TOP_CONTAINER": ".nH.bkK.nn",
					"OUTER_CONTAINER": ".nH .AO",
					"INNER_CONTAINER": ".Tm.aeJ",
					"TO_HIDE": [
						"html:not([data-sortd-sidebar=\"true\"]) .nH .AO .Bu.y3",
						".nH .AO .Bu > div:last-child .l2.ov > div"
					],
					"REMOVE_PADDING_MARGIN": [
						".Tm.aeJ",
						".Bu .nH.if"
					],
					"SCROLLBAR": ".Tm",
					"CSS": [
						".SI .aeF { margin: auto!important; max-width: none!important; }",
						".SI .aDd { display: none!important; }",
						".Su .nH.aBz>:first-child { display: none!important; }",
						".SI .aBz { box-shadow: none!important; }",
						".SI .Tm { background-color: #ffffff!important; }",
						".azY { top: 5px!important; }"
					],
					"ARCHIVE": {
						"SELECTORS": ["[gh=\"tm\"] .G-Ni .lR .asa:not(.sdCompleteArchive)"],
						"REPLACE_CLASS": ["lR"],
						"ICON": {
							"SELECTORS": [".ar8"]
						}
					},
					"MARK_AS_UNREAD": {
						"SELECTORS": [
							"[gh=\"tm\"] .G-Ni .bvt:not(.sortd_dateplan_override) .asa"
						]
					},
					"MARK_AS_STAR": {
						"SELECTORS": [
							"div[role=\"checkbox\"]:not(.sortd_dateplan_override) > .T-KT"
						]
					},
					"MORE_MENU": {
						"BUTTON": {
							"SELECTORS": [
								"[gh=\"tm\"] .G-Ni .nf[role=button]"
							]
						},
						"POP_UP": {
							"SELECTORS": [
								".J-M.aX0.aYO.jQjAxd"
							],
							"MARK_AS_UNREAD": {
								"SELECTORS": [
									".J-N:nth-child(3):not(.sortd_dateplan_override) .J-N-Jz"
								]
							},
							"MARK_AS_STAR": {
								"SELECTORS": [
									".J-N:nth-child(7):not(.sortd_dateplan_override) .J-N-Jz"
								]
							}
						}
					},
					"MESSAGE_MORE_MENU": {
						"BUTTON": {
							"SELECTORS": [
								".T-I.J-J5-Ji.aap[role=button]"
							]
						},
						"POP_UP": {
							"SELECTORS": [
								".b7.J-M[aria-haspopup=true]"
							],
							"MARK_AS_UNREAD": {
								"SELECTORS": [
									".J-N:nth-last-child(1):not(.sortd_dateplan_override) .J-N-Jz"
								]
							}
						}
					},
					"BODY": {
						"SELECTORS": [
							"[role=main] .g.id table.Bs > tr"
						]
					}
				}
			],

			"THREAD_LIST": {
				"ACTION_ITEMS": {
					"ARCHIVE_BUTTON": {
						"SELECTORS": ["[gh=\"tm\"] .G-Ni .lR .asa"]
					}
				}
			},

			"THREAD_ROW": {
				"ACTION_ITEMS": {
					"SELECTORS": ["[role=toolbar]"],
					"ARCHIVE_BUTTON": {
						"SELECTORS": [".brq"]
					}
				}
			},

			"GMAIL_BUTTON_BAR": [
				{
					"ELEMENT": ".nH .D.E.G-atb .nH.aqK .G-tF",
					"MORE_BUTTON": ".T-I.J-J5-Ji.nf"
				}
			],

			"GMAIL_BUTTON_SELECTORS": {
				"BODY": [
					{ "ELEMENT": "body.aAU" }
				],
				"MAIL_SUBJECT": [
					{ "ELEMENT": ".Bu .nH.if .nH .ha .hP" }
				],
				"MAIL_CONVERSATION": [
					{ "ELEMENT": ".Bu .nH.if .h7" }
				],
				"MARK_AS_UNREAD_BUTTON": [
					{
						"ELEMENT": ".J-M.aX0.aYO .J-N .J-N-Jz:contains(\"Mark as unread\")",
						"MORE_MEMU_CONTAINER": ".G-atb .iH .G-Ni.J-J5-Ji .T-I.J-J5-Ji.ar7.nf.T-I-ax7.L3 .Ykrj7b:contains(\"More\")"
					}
				]
			},
			"GMAIL_SEARCH": [
				{
					"ELEMENT": "input[name='q']",
					"ONE_RESULT": ".BltHke.nH.oy8Mbf:not([style*='display:']) .zA:only-child"
				}
			],
			"SEND_MAIL_BUTTON": [
				{
					"BODY_COMMON_PARENT": ".aoP",
					"ELEMENT": ".T-I.J-J5-Ji.aoO"
				}
			],
			"JS_FRAME": [
				{ "ELEMENT": "#js_frame" }
			],

			"LOADER_HIDDEN": [
				{ "ELEMENT": "#loading:hidden" }
			],
			"SIDEBAR": [
				{
					"ELEMENT": "div.AO",
					"TO_HIDE": [
						".nH .AO .Bu.y3",
						".nH .AO .Bu > div:last-child .l2.ov > div"
					],
					"REMOVE_PADDING_MARGIN": [
						".Tm.aeJ",
						".Bu .nH.if"
					],
					"ELEMENT_TO_RESIZE": "div.AO",
					"ELEMENT_TO_RESIZE_IN_CONV_VIEW": "div.AO .aeF > .nH > .nH:not(.BltHke)"
				}
			],
			"GMAIL_REFRESH_BUTTON": [
				{
					"ELEMENT": ".asf.T-I-J3.J-J5-Ji",
					"PARENT": ".D.E.G-atb",
					"MAKE_NOT_VISIBLE": [".G-Ni.J-J5-Ji:not(:has(div.asf))", ".Cr.aqJ"]
				}
			],
			"GMAIL_NOTIFICATION_BAR": [
				{
					"ELEMENT": ".b8.UC",
                    /**
                     * Container: b8 UC (Active: bAp)
                     *    Bar: vh
                     *        Msg Container: aT
                     *            Text: bAq
                     *            Buttons Container: bAo
                     *                Button: ag a8k
                     *        Close Container: bBe
                     *            Button: bBf
                     */
					"CLASSES": {
						"CONTAINER": ["b8", "UC"],
						"CONTAINER_ACTIVE": ["bAp"],
						"BAR": ["vh"],
						"MSG_CONTAINER": ["aT"],
						"MSG_TEXT": ["bAq"],
						"MSG_BUTTON_CONTAINER": ["bAo"],
						"MSG_BUTTONS": ["ag", "a8k"],
						"CLOSE_CONTAINER": ["bBe"],
						"BUTTON": ["bBf"],
						"INBOXSDK_FIXER": ".inboxsdk__size_fixer .aDj.aDi"
					},
					"DONT_SHOW_NOTIFICATION_BAR_SELECTORS": [
						".mixmax-appbar"
					]
				}
			],
			"GMAIL_ADD_ON_BAR": [
				{ "ELEMENT": ".nH.bnl.nn" },
				{ "ELEMENT": ".nH.bAw.nn" }
			],
			"GMAIL_ADD_ON_SORTD_BUTTON": [
				{ "ELEMENT": ".J-KU[data-tooltip=\"Sortd\"]" }
			],
			"GMAIL_ADD_ON": {
				"TOP_BUTTON_CONTAINER": [{ "ELEMENT": ".brC-aT5-aOt-bsf-Jw" }],
				"CONTENT_CONTAINER": [
					{
						"ELEMENT": ".bq9 .brC-brG",
						"PARENT": ".bq9"
					}
				],
				"CONTENT_SIBLING": {
					"ELEMENT": ".nH",
					"CLASS": "sortdAddOnSidebar",
					"RESIZE_CLASS": "sortdSidebarVisible"
				},
				"SORTD_BUTTON": [
					{ "ELEMENT": ".J-KU[data-tooltip=\"Sortd\"]" },
					{ "ELEMENT": ".bse-bvF-I[aria-label=\"Sortd\"]:not(#sortdAddOnButtonTopButtonContainer)" }
				],
				"SORTD_REPLACEMENT_BUTTON": {
					"ID": "sortdAddOnButtonTopButtonContainer",
					"CONTAINER_CLASS": "bse-bvF-I aT5-aOt-I bse-bvF-aLp",
					"CONTAINER_CLASS_HOVER": "aT5-aOt-I-JW",
					"CONTAINER_CLASS_SELECTED": "aT5-aOt-I-KO",
					"CURRENT_ACTIVE_ITEMS": [{
						"ELEMENT": ".aT5-aOt-I-KO",
						"PRIMARY": true
					}, {
						"ELEMENT": ".bse-bvF-I.aT5-aOt-I",
						"PRIMARY": false
					}],
					"HTML_CONTENT": "<div class=\"aT5-aOt-I-JX-Jw\" style=\"background-image: url(&quot;https://staticweb.sortd.com/gmail-addon/squirrel_no_dot.png&quot;); user-select: none;\"></div>\n<div class=\"bse-bvF-JX-axQ-avS\" style=\"background-color: rgb(66, 133, 244); user-select: none;\"></div>",
					"ATTRIBUTES": [
						{ "key": "role", "value": "tab" },
						{ "key": "aria-label", "value": "Sortd" },
						{ "key": "aria-selected", "value": "false" },
						{ "key": "style", "value": "user-select: none" }
					]
				}
			},

            /**
             * Fallbacks for getting the user"s email address
             */
			"GMAIL_ACCOUNT": {
				"GLOBALS_INDEX": 10,
				"ELEMENTS": [
					[
						{
							"element": "title",
							"fn": "html",
							"regex": ".* - (\\S+@\\S+\\.\\S+) - .*"
						}
					], [
						{
							"element": "#loading .msg",
							"fn": "html",
							"regex": ".* (\\S+@\\S+\\..*?)\u2026"
						}
					], [
						{
							"element": ".gb_fa .gb_l",
							"fn": "html",
							"regex": "(\\S+@\\S+\\.\\S+)"
						}
					]
				]
			},
			"EXTENSIONS": {
				"RAPPORTIVE": [
					{
						"ELEMENT": "html[data-rapportive-launchpad=\"true\"]"
					}
				],
				"INBOXSDK": [
					{
						"ELEMENT": "html[data-inboxsdk-active-app-ids]"
					}
				],
				"SIDEKICK": {
					"CONTAINER": ".signals-insights-container",
					"OPEN": [
						{
							"ELEMENT": "body.insights-user-shown"
						}
					],
					"CLOSE": [
						{
							"ELEMENT": "body:not(.insights-user-shown)"
						}
					]
				}
			},
			"GMAIL_TABS": {
				"GLOBALS": [{
					"INDEX": [17, 4, 1],
					"TAB_PROP_NAME": "sx_pits",
					"STAR_PROP_NAME": "bx_msis",
					"LABEL_NAMES": [
						{ "smartlabel_personal": "CATEGORY_PERSONAL" },
						{ "smartlabel_social": "CATEGORY_SOCIAL" },
						{ "smartlabel_promo": "CATEGORY_PROMOTIONS" },
						{ "smartlabel_notification": "CATEGORY_UPDATES" },
						{ "smartlabel_group": "CATEGORY_FORUMS" }
					],
					"LABEL_STARRED": "STARRED"
				}, {
					"INDEX": [17, 4, 1],
					"TAB_PROP_NAME": "sx_iosc",
					"STAR_PROP_NAME": "bx_msis",
					"LABEL_NAMES": [
						{ "smartlabel_personal": "CATEGORY_PERSONAL" },
						{ "smartlabel_social": "CATEGORY_SOCIAL" },
						{ "smartlabel_promo": "CATEGORY_PROMOTIONS" },
						{ "smartlabel_notification": "CATEGORY_UPDATES" },
						{ "smartlabel_group": "CATEGORY_FORUMS" }
					],
					"LABEL_STARRED": "STARRED"
				}],
				"SELECTOR_TEMPLATE": "[role=\"main\"] [role=\"tablist\"] .${LABEL_SELECTOR}-aLe[role=\"tab\"]",
				"LABELS": [
					{ "name": "CATEGORY_PERSONAL", "selector": "aIf" },
					{ "name": "CATEGORY_SOCIAL", "selector": "aKe" },
					{ "name": "CATEGORY_PROMOTIONS", "selector": "aJi" },
					{ "name": "CATEGORY_UPDATES", "selector": "aH2" },
					{ "name": "CATEGORY_FORUMS", "selector": "aHE" }
				]
			},

            /**
             * Other DOM elements or classes that do no require fallbacks
             */
			"VIEW_MAIL_POP_UP": {
				"URL_CONTAINS_DOMAIN": "mail.google",
				"URL_CONTAINS_QRY_STR": "view=btop"
			},

			"NAV_MENU": {
				"UNREAD_COUNTER": {
					"SELECTORS": [
						".bsU"
					]
				}
			}
		};
	}
})();
