/**
 * Sortd
 * Copyright (c) 2019, Sortd Inc
 * https://www.sortd.com
 *
 * @fileoverview Perform http requests to the server
 * @name sortd-http
 */

var SortdHttp = (function () {

	'use strict';

	/**
	 * Initialize the class
	 * @class SortdHttp
	 */
	function SortdHttp () {

		this.FILE_NAME = 'sortd-http.js';

		this.messages = SortdMessages.getInstance();
	}

	/**
	 * Pass a GET request to the background page
	 * @param {String} url - The url to get
	 * @param {Object} [params] - Optional. JSON object with key:value pairs to send with GET request
	 * @param {Function} next - The results callback
	 * @private
	 */
	SortdHttp.prototype.get = function (url, params, next, retries) {

		this.messages.sendMessageToEventPage(
			'httpGet',
			{ url, params, retries },
			(err, response) => next(err || null, response && response.result));
	};

	/**
	 * Pass a POST request to the background page
	 * @param {String} url - The url to post to
	 * @param {Object} [params] - Optional. JSON object with key:value pairs to send with POST request
	 * @param {Function} next - The results callback
	 * @private
	 */
	SortdHttp.prototype.post = function (url, params, next) {

		this.messages.sendMessageToEventPage(
			'httpPost',
			{ url, params },
			(err, response) => next(err || null, response && response.result));
	};

	return SortdHttp;
}());